/*********************************************************************/
/* brail.c      v 1.0      by J.Sobczyk IAPW       1995.06.24  11:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* Brail encoding.                                                   */
/*-------------------------------------------------------------------*/
/* Kodowanie liter: a, -> '1', c' -> '3', e, -> '5', l- -> '2',      */
/*       n' -> '4', o' -> '0', s' -> '9', z. -> '&', z' -> '^'.      */
/* Duze litery poprzedza sie znakeim '$'.                            */
/* Ciag duzych liter poprzedza sie sekwencja "$$".                   */
/* Cyfry koduje sie: 1 -> 'A', 2 -> 'B', ... 9 -> 'I', 0 -> 'J'      */
/* poprzedzajac je znakiem: '#'.                                     */
/* Kodowanie znakow specjalnych: '.' -> '/', '!' -> '+', '(' -> '=', */
/* ')' -> '=', '"' otwierajacy -> '(', '"' zamykajacy -> ')'.        */
/* Nowa linie w wierszach oznacza sie znakiem: '@'.                  */
/* Ciagi spacji nalezy redukowac do jednej spacji.                   */
/* Ciagi pustych linii nalezy skracac do jedej pustej linii.         */
/*********************************************************************/
/* Zmainy:                                                           */
/* 1995.12.5 Ciag duzych liter poprzedza sie znakiem "$".            */
/* 1995.12.5 Ciag cyfr poprzedza sie jednym znakiem "#".             */
/*********************************************************************/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
code_tbl brail_tbl = /* Brail */
  {
    { '1',  'a' },
    { '3',  'c' },
    { '5',  'e' },
    { '2',  'l' },
    { '4',  'n' },
    { '0',  'o' },
    { '9',  's' },
    { '^',  'x' },
    { '&',  'z' },
    { '1',  'A' },
    { '3',  'C' },
    { '5',  'E' },
    { '2',  'L' },
    { '4',  'N' },
    { '0',  'O' },
    { '9',  'S' },
    { '^',  'X' },
    { '&',  'Z' }
  };

/*-------------------------------------------------------------------*/
int brail_qstr	= 0;	/* True inside of quoted string: "....."     */
int brail_level = 0;	/* Nesting level for parenthesis.            */
int brail_space	= 0;	/* True after whitespace.                    */
int brail_line	= 0;	/* Number of consecutive new line.           */
int brail_cap	= 0;	/* True after capital letter.                */
int brail_dig	= 0;	/* True after a digit.                       */

/*-------------------------------------------------------------------*/
long int brail_hits  = 0;
long int brail_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int brail_get( void )
  {
  int	i;
  int	ch;
  static int	ch1 = 0;

  if (ch1)
    {
    ch = ch1;
    ch1 = 0;
    }
  else
    ch = getc( inp );
  
  switch (ch)
    {
    case '/':
      brail_cap = 0;
      return '.';
    case '+':
      brail_cap = 0;
      return '!';
    case '(':
    case ')':
      brail_cap = 0;
      return '"';
    case '=':
      brail_cap = 0;
      if (brail_level == 0)
	{
	brail_level++;
	return '(';
	}
      else
	{
	brail_level--;
	return ')';
	}
    case '#':
      brail_cap = 0;
      brail_dig = 1;
      ch = getc( inp );
      if (ch == 'J')
	i = '0';
      else
	i = ch - 'A' + '1';
      return i;
    case '$':
//      ch = getc( inp );
//      if (ch == '$')
//	{
	brail_cap = 1;
//	ch = getc( inp );
//	}
//      ch1 = ch;
      ch = brail_get();		// recursion!!
      brail_cap = 0;
      return ch;
//      return toupper( ch );
    default:
      if (brail_dig && ch >= 'A' && ch <= 'J')
	{
	if (ch == 'J')
	  i = '0';
	else
	  i = ch - 'A' + '1';
	return i;
	}
      else if (brail_dig)
	{
	brail_dig = 0;
	}
      for (i=0; i< 2*POL_NR; i++)
	if ((char) ch == brail_tbl[i][0])
	  {
	  if (brail_cap)
	    return POL_CHAR | toupper(brail_tbl[i][1]);
	  else
	    return POL_CHAR | tolower(brail_tbl[i][1]);
	  }
      if (!isalpha( ch ))
	brail_cap = 0;
    }
  if (brail_cap)
    return toupper( ch );
  else
    return tolower( ch );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void brail_put( int ch )
  {
  int		i;

  if (ch == EOF)
    ;
  else if (ch & POL_CHAR)	/* Process polish characters */
    {
    ch &= 0x0FF;
    if (isupper( ch ) && !brail_cap)
      {
      fputc( '$', out );
      }
    for (i=0; i< 2*POL_NR; i++)
      if ((char) ch == brail_tbl[i][1])
	{
	fputc( brail_tbl[i][0], out );
	}
    brail_space = 0;
    brail_line	= 0;
    brail_cap	= isupper( ch );
    brail_dig	= isdigit( ch );
    }
  else
    {
    switch (ch)
      {
      case '.':
	fputc( '/', out );
	break;
      case '!':
	fputc( '+', out );
	break;
      case '"':
	if (brail_qstr)
	  {
	  fputc( ')', out );
	  brail_qstr = 0;
	  }
	else
	  {
	  fputc( '(', out );
	  brail_qstr = 1;
	  }
	break;
      case '(':
      case ')':
	fputc( '=', out );
	break;
      case ' ':
      case '\t':
	if (!brail_space)
	  fputc( ' ', out );
	break;
      case '\n':
	if (brail_line < 2)
	  fputc( '\n', out );
	break;
      default:
	if (isupper( ch ) && !brail_cap)
	  {
	  fputc( '$', out );
	  fputc( ch, out );
	  }
	else if (isdigit( ch ))
	  {
	  if (!brail_dig)
	    {
	    fputc( '#', out );
	    }
	  if (ch == '0')
	    i = 'J';
	  else
	    i = ch - '1' + 'A';
	  fputc( i, out );
	  }
	else
	  fputc( ch, out );
      }
    brail_space = (ch == ' ') || (ch == '\t');
    brail_cap	= isupper( ch );
    brail_dig	= isdigit( ch );
    if (ch == '\n')
      brail_line++;
    else
      brail_line = 0;
    }

  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_brail( char *name )
  {
  if      (inp_type == TYPE_UNSPECIFIED)
    {
    inp_type   = TYPE_BRAIL;
    inp_name   = name;
    inp_get    = brail_get;
    }
  else if (out_type == TYPE_UNSPECIFIED)
    {
    out_type   = TYPE_BRAIL;
    out_name   = name;
    out_put    = brail_put;
    }
  else
    {
    fprintf( stderr, "Too many switches\a\n" );
    help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_brail( char *buff )
  {
  int i;

  if (buff[0] & 0x80)
    {
    brail_fails++;
    return;
    }

  for (i=0; i< 2*POL_NR; i++)
    if (  (  ((char) buff[0] == brail_tbl[i][0]) && isalpha(buff[1]) ) ||
          (  ((char) buff[1] == brail_tbl[i][0]) && isalpha(buff[0]) ) )
      {
      brail_hits++;
      return;
      }
    else if (  ((char) buff[0] == brail_tbl[i][0]) ||
               ((char) buff[1] == brail_tbl[i][0]) )
      {
      brail_fails++;
      return;
      }

  if ((buff[0] == '#') && (tolower(buff[1]) >= 'a') &&
			  (tolower(buff[1]) <= 'j')    )
    {
    brail_hits++;
    return;
    }

  brail_fails++;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_brail( void )
  {
  return brail_hits-brail_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_brail( void )
  {
  tests[ TYPE_BRAIL ].key		= "-b";
  tests[ TYPE_BRAIL ].name		= "Brail";
  tests[ TYPE_BRAIL ].set		= set_brail;
  tests[ TYPE_BRAIL ].char_tst	= test_char_brail;
  tests[ TYPE_BRAIL ].freq_tst	= test_freq_brail;
  }

/*********************************************************************/
