/*********************************************************************/
/* file.c       v 1.0      by J.Sobczyk IAPW       1994.12.05  19:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* User defined conversion.                                          */
/*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
code_tbl file_tbl = /* user defined */
  {
    { '\xB1',  'a' },	/* Default values ISO 8859-2 */
    { '\xE6',  'c' },
    { '\xEA',  'e' },
    { '\xB3',  'l' },
    { '\xF1',  'n' },
    { '\xF3',  'o' },
    { '\xB6',  's' },
    { '\xBC',  'x' },
    { '\xBF',  'z' },
    { '\xA1',  'A' },
    { '\xC6',  'C' },
    { '\xCA',  'E' },
    { '\xA3',  'L' },
    { '\xD1',  'N' },
    { '\xD3',  'O' },
    { '\xA6',  'S' },
    { '\xAC',  'X' },
    { '\xAF',  'Z' }
  };

/*-------------------------------------------------------------------*/
struct
  {
  char code[3];
  char key;
  } user_codes[2*POL_NR] =
  {
  { "a,", 'a' },
  { "c'", 'c' },
  { "e,", 'e' },
  { "l-", 'l' },
  { "n'", 'n' },
  { "o'", 'o' },
  { "s'", 's' },
  { "z'", 'x' },
  { "z.", 'z' },
  { "A,", 'A' },
  { "C'", 'C' },
  { "E,", 'E' },
  { "L-", 'L' },
  { "N'", 'N' },
  { "O'", 'O' },
  { "S'", 'S' },
  { "Z'", 'X' },
  { "Z.", 'Z' }
  };

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void load_table( char *table )
  {
  static char line[200];
  FILE *inp;
  int i;
  char *val;
  unsigned char code;
  
  inp = fopen( table, "r" );
  if (inp == NULL)
    {
    fprintf( stderr, "Can not open table definition\n\a" );
    exit( 1 );
    }

  while (fgets( line, 200, inp ) != NULL)
    {
    for (i=0; line[i] != '\0'; i++)
      if (isspace(line[i]))
	{
	strcpy( line+i, line+i+1 );
	i--;
	}
      else if (line[i] == '#')
	{
	line[i] = '\0';
	i--;
	}
	
    val = strtok( line, "=" );
    val = strtok( NULL, "=" );
    if (val == NULL)
      {
      fprintf( stderr, "No value for keyword: %s - skipping\n", line );
      continue;
      }
    code = (unsigned char) strtol( val, NULL, 0 );

    for (i=0; i<2*POL_NR; i++)
      if (!strcmp( user_codes[i].code, line ))
	break;
	
    if (i < 2*POL_NR)
      {
      file_tbl[i][0] = code;
      file_tbl[i][1] = user_codes[i].key;
      }
    else
      fprintf( stderr, "Unknown keyword: %s - ignored\n", line );
    }

  fclose( inp );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void dump_table( void )
  {
  int i, j;

  if (inp_get != table_get)
    fprintf( stderr, "Not single character code!\n" );

  for (i=0; i < 2*POL_NR; i++)
    {
    printf( "%s = ", user_codes[i].code );
    for (j=0; j < 2*POL_NR; j++)
      if ((*inp_tbl)[j][1] == user_codes[i].key)
	break;
    printf( "0x%0X # %d %c\n",
	    (*inp_tbl)[j][0] & 0xFF,
		  (*inp_tbl)[j][0] & 0xFF,
		      (*inp_tbl)[j][0] );
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_file( char *table, char *name )
  {
  load_table( table );
  if      (inp_type == TYPE_UNSPECIFIED)
    {
    inp_type   = TYPE_FILE;
    inp_name   = name;
    inp_get    = table_get;
    inp_tbl    = &file_tbl;
    }
  else if (out_type == TYPE_UNSPECIFIED)
    {
    out_type   = TYPE_FILE;
    out_name   = name;
    out_put    = table_put;
    out_tbl    = &file_tbl;
    }
  else
    {
    fprintf( stderr, "Too many switches\a\n" );
    help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_file( void )
  {
  return table_test( &file_tbl );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_file( void )
  {
  tests[ TYPE_FILE ].key	= "-f file";
  tests[ TYPE_FILE ].name	= "user defined";
  tests[ TYPE_FILE ].set	= NULL; /* Exception - called indirectly */
  tests[ TYPE_FILE ].char_tst	= NULL;
  tests[ TYPE_FILE ].freq_tst	= NULL;
  }

/*********************************************************************/
