/*********************************************************************/
/* iso10.c      v 1.0      by J.Sobczyk IAPW       1995.10.20  19:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* ISO 10646                                                         */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
char iso_10646_tbl[2*POL_NR][3] = /* ISO 10646 */
  {
    { '\x21', '\xb1',	'a' },	/* <a;>  LATIN SMALL LETTER A WITH OGONEK */
    { '\x21', '\xe6',	'c' },	/* <c'>  LATIN SMALL LETTER C WITH ACUTE */
    { '\x21', '\xea',	'e' },	/* <e;>  LATIN SMALL LETTER E WITH OGONEK */
    { '\x21', '\xb3',	'l' },	/* <l//> LATIN SMALL LETTER L WITH STROKE */
    { '\x21', '\xf1',	'n' },	/* <n'>  LATIN SMALL LETTER N WITH ACUTE */
    { '\x20', '\xf3',	'o' },	/* <o'>  LATIN SMALL LETTER O WITH ACUTE */
    { '\x21', '\xb6',	's' },	/* <s'>  LATIN SMALL LETTER S WITH ACUTE */
    { '\x21', '\xbc',	'x' },	/* <z'>  LATIN SMALL LETTER Z WITH ACUTE */
    { '\x21', '\xbf',	'z' },	/* <z.>  LATIN SMALL LETTER Z WITH DOT ABOVE */
    { '\x21', '\xa1',	'A' },	/* <A;>  LATIN CAPITAL LETTER A WITH OGONEK */
    { '\x21', '\xc6',	'C' },	/* <C'>  LATIN CAPITAL LETTER C WITH ACUTE */
    { '\x21', '\xca',	'E' },	/* <E;>  LATIN CAPITAL LETTER E WITH OGONEK */
    { '\x21', '\xa3',	'L' },	/* <L//> LATIN CAPITAL LETTER L WITH STROKE */
    { '\x21', '\xd1',	'N' },	/* <N'>  LATIN CAPITAL LETTER N WITH ACUTE */
    { '\x20', '\xd3',	'O' },	/* <O'>  LATIN CAPITAL LETTER O WITH ACUTE */
    { '\x21', '\xa6',	'S' },	/* <S'>  LATIN CAPITAL LETTER S WITH ACUTE */
    { '\x21', '\xac',	'X' },	/* <Z'>  LATIN CAPITAL LETTER Z WITH ACUTE */
    { '\x21', '\xaf',	'Z' },	/* <Z.>  LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  };

long int iso_10646_hits  = 0;
long int iso_10646_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int iso10_get( void )
  {
  int	ch0;
  int	ch1;
  int	ch2;
  int	ch3;
  int	i;

  ch0 = getc( inp );
  if (ch0 == EOF)
    return ch0;

  ch1 = getc( inp );
  if (ch1 == EOF)
    return ch0;

  ch2 = getc( inp );
  if (ch2 == EOF)
    return ch0;

  ch3 = getc( inp );
  
  for (i=0; i< 2*POL_NR; i++)
    if (((char) ch0 == '\x20') &&
        ((char) ch1 == '\x20') &&
        ((char) ch2 == iso_10646_tbl[i][0]) &&
        ((char) ch3 == iso_10646_tbl[i][1])   )
      {
      return POL_CHAR | iso_10646_tbl[i][2];
      }

  if ((ch0 != '\x20') || (ch1 != '\x20') || (ch2 != '\x20'))
    {
    fprintf( stderr, "Character %02x%02x%02x%02x changed into @\n", ch0, ch1, ch2, ch3 );
    ch3 = '@';
    }
  return ch3;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void iso10_put( int ch )
  {
  int i;

  if (ch == EOF)
    ;
  else if (ch & POL_CHAR)
    {
    ch &= 0x0FF;
    for (i=0; i< 2*POL_NR; i++)
      if ((char) ch == iso_10646_tbl[i][2])
	{
	fputc( '\x20', out );
	fputc( '\x20', out );
	fputc( iso_10646_tbl[i][0], out );
	fputc( iso_10646_tbl[i][1], out );
	}
    }
  else
    {
    fputc( '\x20', out );
    fputc( '\x20', out );
    fputc( '\x20', out );
    fputc( ch, out );
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_iso_10646( char *name )
  {
  if      (inp_type == TYPE_UNSPECIFIED)
    {
    inp_type   = TYPE_ISO_10646;
    inp_name   = name;
    inp_get    = iso10_get;
    }
  else if (out_type == TYPE_UNSPECIFIED)
    {
    out_type   = TYPE_ISO_10646;
    out_name   = name;
    out_put    = iso10_put;
    }
  else
    {
    fprintf( stderr, "Too many switches\a\n" );
    help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_iso_10646( char *buff )
  {
  int i;

  for (i=0; i< 2*POL_NR; i++)
    if (((char) buff[0] == '\x20') &&
        ((char) buff[1] == '\x20') &&
        ((char) buff[2] == iso_10646_tbl[i][0]) &&
        ((char) buff[3] == iso_10646_tbl[i][1])   )
      {
      iso_10646_hits++;
      return;
      }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_iso_10646( void )
  {
  return iso_10646_hits-iso_10646_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_iso_10646( void )
  {
  tests[ TYPE_ISO_10646 ].key		= "-i10";
  tests[ TYPE_ISO_10646 ].name		= "ISO 10646";
  tests[ TYPE_ISO_10646 ].set		= set_iso_10646;
  tests[ TYPE_ISO_10646 ].char_tst	= test_char_iso_10646;
  tests[ TYPE_ISO_10646 ].freq_tst	= test_freq_iso_10646;
  }

/*********************************************************************/
