/*********************************************************************/
/* iso6.c       v 1.0      by J.Sobczyk IAPW       1993.04.26  19:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* ISO 6937-2                                                        */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
char iso_6937_2_tbl[2*POL_NR-2][3] = /* ISO 6937-2 */
  {
    { '\x61', '\xCE',  'a' },
    { '\x63', '\xC2',  'c' },
    { '\x65', '\xCE',  'e' },
    { '\x6E', '\xC2',  'n' },
    { '\x6F', '\xC2',  'o' },
    { '\x73', '\xC2',  's' },
    { '\x7A', '\xC2',  'x' },
    { '\x7A', '\xC7',  'z' },
    { '\x41', '\xCE',  'A' },
    { '\x43', '\xC2',  'C' },
    { '\x45', '\xCE',  'E' },
    { '\x4E', '\xC2',  'N' },
    { '\x4F', '\xC2',  'O' },
    { '\x54', '\xC2',  'S' },
    { '\x5A', '\xC2',  'X' },
    { '\x5A', '\xC7',  'Z' }
  };

/*-------------------------------------------------------------------*/
char iso_6937_2_tbx[2][2] = /* ISO 6937-2 */
  {
    { '\xF8',  'l' },
    { '\xE8',  'L' }
  };

long int iso_6937_2_hits  = 0;
long int iso_6937_2_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int iso_get( void )
  {
  static first = 1;
  static int	ch1;
  int		ch0;
  int		i;

  if (first)
    {
    ch1 = getc( inp );
    first = 0;
    }
  ch0 = ch1;

  if (ch0 == EOF)
    return ch0;

  ch1 = getc( inp );
  
  for (i=0; i< 2; i++)
    if ((char) ch0 == iso_6937_2_tbx[i][0])
      return POL_CHAR | iso_6937_2_tbx[i][1];
  for (i=0; i< 2*POL_NR-2; i++)
    if (((char) ch0 == iso_6937_2_tbl[i][0]) &&
        ((char) ch1 == iso_6937_2_tbl[i][1])   )
      {
      ch1 = getc( inp );
      return POL_CHAR | iso_6937_2_tbl[i][2];
      }

  return ch0;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void iso_put( int ch )
  {
  int i;

  if (ch == EOF)
    ;
  else if (ch & POL_CHAR)
    {
    ch &= 0x0FF;
    for (i=0; i< 2*POL_NR-2; i++)
      if ((char) ch == iso_6937_2_tbl[i][2])
	{
	fputc( iso_6937_2_tbl[i][0], out );
	fputc( iso_6937_2_tbl[i][1], out );
	}
    for (i=0; i< 2; i++)
      if ((char) ch == iso_6937_2_tbx[i][1])
	fputc( iso_6937_2_tbx[i][0], out );
    }
  else
    fputc( ch, out );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_iso_6937_2( char *name )
  {
  if      (inp_type == TYPE_UNSPECIFIED)
    {
    inp_type   = TYPE_ISO_6937_2;
    inp_name   = name;
    inp_get    = iso_get;
    }
  else if (out_type == TYPE_UNSPECIFIED)
    {
    out_type   = TYPE_ISO_6937_2;
    out_name   = name;
    out_put    = iso_put;
    }
  else
    {
    fprintf( stderr, "Too many switches\a\n" );
    help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_iso_6937_2( char *buff )
  {
  int i;

  for (i=0; i< 2; i++)
    if (buff[0] == iso_6937_2_tbx[i][0])
      {
      iso_6937_2_hits++;
      return;
      }
      
  for (i=0; i< 2*POL_NR-2; i++)
    if (((char) buff[0] == iso_6937_2_tbl[i][0]) &&
        ((char) buff[1] == iso_6937_2_tbl[i][1])   )
      {
      iso_6937_2_hits++;
      return;
      }

  if (buff[0] & 0x80)
    iso_6937_2_fails++;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_iso_6937_2( void )
  {
  return iso_6937_2_hits-iso_6937_2_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_iso_6937_2( void )
  {
  tests[ TYPE_ISO_6937_2 ].key		= "-i6";
  tests[ TYPE_ISO_6937_2 ].name		= "ISO 6937-2";
  tests[ TYPE_ISO_6937_2 ].set		= set_iso_6937_2;
  tests[ TYPE_ISO_6937_2 ].char_tst	= test_char_iso_6937_2;
  tests[ TYPE_ISO_6937_2 ].freq_tst	= test_freq_iso_6937_2;
  }

/*********************************************************************/
