/*********************************************************************/
/* polcnv.c     v 1.0      by J.Sobczyk IAPW       1993.04.26  19:00 */
/*                                                                   */
/* Polish document converter.                                        */
/*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "polcnv.h"
#ifdef SVR4
#define stricmp( S1, S2 ) strcasecmp( S1, S2 )
#define strnicmp( S1, S2, LEN ) strncasecmp( S1, S2, LEN )
#endif

/*-------------------------------------------------------------------*/
int  inp_type   = TYPE_UNSPECIFIED;
char inp_prefix = '\0';
char *inp_name  = (char *) NULL;
FILE *inp       = (FILE *) NULL;
int  (*inp_get)( void );
code_tbl *inp_tbl;

/*-------------------------------------------------------------------*/
int  out_type   = TYPE_UNSPECIFIED;
char out_prefix = '\0';
char *out_name  = (char *) NULL;
FILE *out       = (FILE *) NULL;
void (*out_put)( int ch );
code_tbl *out_tbl;

/*-------------------------------------------------------------------*/
test_table tests[ MAX_TEST ];
int verbose_flag = 0;
int dump_flag = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void convert( void )
  {
  int ch;

  if (verbose_flag > 1)
    {
    if (inp_type == TYPE_PREFIX)
      fprintf( stderr, "Converting from %s using %c ",
				 tests[ inp_type ].name, inp_prefix );
    else
      fprintf( stderr, "Converting from %s ", tests[ inp_type ].name );
    if (inp_type == TYPE_PREFIX)
      fprintf( stderr, "to %s using %c ",
				 tests[ out_type ].name, out_prefix );
    else
      fprintf( stderr, "to %s\n", tests[ out_type ].name );
    }

  if (!strcmp(inp_name, "-"))
    inp = stdin;
  else
    {
    inp = fopen( inp_name, "r" );
    if (inp == NULL)
      {
      fprintf( stderr, "Can not open input file: %s\a\n", inp_name );
      exit( 2 );
      }
    }

  if (!strcmp(out_name, "-"))
    out = stdout;
  else
    {
    out = fopen( out_name, "w" );
    if (out == NULL)
      {
      fprintf( stderr, "Can not open output file: %s\a\n", out_name );
      exit( 2 );
      }
    }

  ch = inp_get();
  while (ch != EOF)
    {
    out_put( ch );
    ch = inp_get();
    }
  out_put( EOF );	/* Flush internal buffers! */
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void help( void )
  {
  int t;

  printf( "POLcnv  v1.3  by J.Sobczyk " __DATE__ "\n" );

  printf( "\n"
          "Usage:\n"
	  "    POLcnv [options] inputtype inputfile outputtype outputfile\n"
	  "or:\n"
	  "    POLcnv [options] -t inputfile\n"
	  "Types:\n"
	  "    -p@  - prefix notation using @\n"
	);

  for (t=0; t<MAX_TEST; t++)
    if (t != TYPE_PREFIX)
      printf( "%7s  - %s\n", tests[t].key, tests[t].name );

  printf( "Options:\n"
	  "    -?   - Display this help\n"
	  "    -d   - Dump input conversion table\n"
	  "    -v   - Increase verbosity level\n"
	  "Hyphen used in place of file specification denotes\n"
	  "standard input or output stream.\n"
	  "Use: POLcnv -v -? for more explanations.\n"
	);

  if (verbose_flag > 0)
    {
    printf( "Examples:\n"
	    "  POLcnv -m aaa.txt -i bbb.doc\n"
	    "    convert file aaa.txt coded using Mazowia to bbb.doc using"
	                                                      "ISO 8859-2\n"
	    "  POLcnv -f my.cod abc.txt -i def.txt\n"
	    "    Convert file abc.txt from code defined with my.cod to ISO 8859-2\n"
	    "  POLcnv -d -i\n"
	    "    Dump ISO 8859-2 code table\n"
	    "  POLcnv -t aaa.txt -m ccc.txt\n"
	    "    identify code used for file aaa.txt and covert it to Mazovia\n"
	    "  POLcnv -t xxx.txt\n"
	    "    identify code used for coding file xxx.txt\n"
	    "  POLcnv -v -v -t xxx.txt\n"
	    "    identify code used for coding file xxx.txt and display gathered"
								  " statistics\n"
	    "Verbosity levels:\n"
	    "  0 - error reports only, report only best result for -t\n"
	    "  1 - all positive test results\n"
	    "  2 - dump non zero frequency table entries for character codes"
								  " 0x80..0xff\n"
	    "      and prefix frequencies for non alphabetic and non space"
	    " characters\n"
	    "  3 - dump all non zero frequency table entries and prefix frequencies"
	    " for all characters\n"
	  );
    }

  exit( 1 );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init( void )
  {
  int i;

  for (i=0; i < MAX_TEST; i++)
    {
    tests[ i ].key	= "";
    tests[ i ].name	= "";
    tests[ i ].set	= NULL;
    tests[ i ].char_tst	= NULL;
    tests[ i ].freq_tst	= NULL;
    }
  init_test();
  init_prefix();
  init_mazowia();
  init_latin_2();
  init_iso_8859_2();
  init_windows();
  init_microvex();
  init_unia();
  init_iso_6937_2();
  init_brail();
  init_unicode();
  init_iso_10646();
  init_macintosh();
  init_file();
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int main( int narg, char *argv[] )
  {
  int i;
  int t;

  init();

  for (i=1; i<narg; i++)
    {
    if      (!stricmp( argv[i], "-?" ))
      help();
    else if (!strnicmp( argv[i], "-v", 2 ))
      verbose_flag++;
    else if (!strnicmp( argv[i], "-d", 2 ))
      dump_flag++;
    else if (!strnicmp( argv[i], "-p", 2 ))
      {
      set_prefix( argv[i][2], argv[i+1] );
      i++;
      }
    else if (!strnicmp( argv[i], "-f", 2 ))
      {
      set_file( argv[i+1], argv[i+2] );
      i+=2;
      }
    else
      {
      for (t=0; t<MAX_TEST; t++)
	{
	if (!stricmp( argv[i], tests[t].key ))
	  {
	  if (tests[t].set != NULL)
	    {
	    (*tests[t].set)( argv[i+1] );
	    i++;
	    }
	  break;
	  }
	}
      if (t == MAX_TEST)
	{
	fprintf( stderr, "invalid switch: %s\a\n", argv[i] );
	help();
	}
      }
    }

  if (dump_flag)
    {
    dump_table();
    exit( 0 );
    }

  if (inp_name == NULL)
    {
    fprintf( stderr, "No input file specified\a\n" );
    help();
    }

  if (inp_type == TYPE_TEST)
    run_test();

  if (out_name == NULL)
    exit( 1 );

  if ((inp_type != TYPE_TEST) && (inp_type != TYPE_UNSPECIFIED))
    convert();
  }

/*********************************************************************/
