/*********************************************************************/
/* prefix.c     v 1.0      by J.Sobczyk IAPW       1993.04.26  19:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* Prefix notation                                                   */
/*********************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
char prefix_tbl[] = "aAcCeElLnNoOsSzZxX";

long int pfreq[ 256 ];

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int prefix_get( void )
  {
  int ch;
  int i;

  ch = getc( inp );
  if (ch == inp_prefix)
    {
    ch = getc( inp );
    if (ch == inp_prefix)
      return ch;
    for (i=0; i< 2*POL_NR; i++)
      if ((char) ch == prefix_tbl[i])
	return POL_CHAR | ch;
    fprintf( stderr, "Invalid sequence: %c%c\a\n", inp_prefix, (char)ch );
    return ch;
    }
  return ch;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void prefix_put( int ch )
  {
  if (ch == EOF)
    ;
  else if (ch == out_prefix)
    {
    fputc( out_prefix, out );
    fputc( out_prefix, out );
    }
  else if (ch & POL_CHAR)
    {
    ch &= 0x0FF;
    fputc( out_prefix, out );
    fputc( ch, out );
    }
  else
    fputc( ch, out );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_prefix( char prefix, char *name )
  {
  if      (inp_type == TYPE_UNSPECIFIED)
    {
    inp_type   = TYPE_PREFIX;
    inp_name   = name;
    inp_prefix = prefix;
    inp_get    = prefix_get;
    }
  else if (out_type == TYPE_UNSPECIFIED)
    {
    out_type   = TYPE_PREFIX;
    out_name   = name;
    out_prefix = prefix;
    out_put    = prefix_put;
    }
  else
    {
    fprintf( stderr, "Too many switches\a\n" );
    help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_prefix( char *buff )
  {
  int i;

  for (i=0; i< 2*POL_NR; i++)
    if (buff[1] == prefix_tbl[i])
      pfreq[ buff[0] & 0xFF ]++;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_prefix( void )
  {
  int ch;
  int best;
  int best_ch;

  best    = -1;
  best_ch = -1;
  for (ch=0; ch<256; ch++)
    {
    if (isalpha(ch) || isspace(ch))
      {
      if ((verbose_flag > 2) && (pfreq[ch] != 0))
	printf( "Prefix[%02x]=%5ld\n", ch, pfreq[ch] );
      }
    else 
      {
      if (pfreq[ch] > best)
	{
	best_ch = ch;
	best    = pfreq[ch];
	}

      if ((verbose_flag > 1) && (pfreq[ch] != 0))
	printf( "Prefix[%02x]=%5ld\n", ch, pfreq[ch] );
      }
    }

  inp_prefix = best_ch;
  return best;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_prefix( void )
  {
  int i;

  for (i=0; i<256; i++)
    pfreq[i] = 0;

  tests[ TYPE_PREFIX ].key	= "-p@";
  tests[ TYPE_PREFIX ].name	= "Prefix notation";
  tests[ TYPE_PREFIX ].set	= NULL; /* Exception - called indirectly */
  tests[ TYPE_PREFIX ].char_tst	= test_char_prefix;
  tests[ TYPE_PREFIX ].freq_tst	= test_freq_prefix;
  }

/*********************************************************************/
