/*********************************************************************/
/* test.c       v 1.0      by J.Sobczyk IAPW       1994.01.27  19:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* Character code detector.                                          */
/*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
code_tbl test_tbl = /* Latin 2 ISO 8859-2 */
  {
    { '\xB1',  'a' },
    { '\xE6',  'c' },
    { '\xEA',  'e' },
    { '\xB3',  'l' },
    { '\xF1',  'n' },
    { '\xF3',  'o' },
    { '\xB6',  's' },
    { '\xBC',  'x' },
    { '\xBF',  'z' },
    { '\xA1',  'A' },
    { '\xC6',  'C' },
    { '\xCA',  'E' },
    { '\xA3',  'L' },
    { '\xD1',  'N' },
    { '\xD3',  'O' },
    { '\xA6',  'S' },
    { '\xAC',  'X' },
    { '\xAF',  'Z' }
  };

long int frequency[ 256 ];
char buffer[ MAX_BUFF ];

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int  test_get( void )
  {
  fprintf( stderr, "Test failed\n" );
  exit( 1 );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_put( int ch )
  {
  fprintf( stderr, "Test failed\n" );
  exit( 1 );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_test( char *name )
  {
  if      (inp_type == TYPE_UNSPECIFIED)
    {
    inp_type   = TYPE_TEST;
    inp_name   = name;
    inp_get    = test_get;
    inp_tbl    = &test_tbl;
    }
  else if (out_type == TYPE_UNSPECIFIED)
    {
    fprintf( stderr, "Test can not be used on output\a\n" );
    help();
    }
  else
    {
    fprintf( stderr, "Too many switches\a\n" );
    help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void run_chr_tsts( int nr )
  {
  int t;

  for (t=0; t<MAX_TEST; t++)
    if (tests[ t ].char_tst != NULL)
      (*tests[ t ].char_tst)( buffer+nr-MAX_SEQ );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void shift_buffer( int nr )
  {
  int t;

  for (t=0; t<MAX_SEQ; t++)
    buffer[ t ] = buffer[ t + nr - MAX_SEQ ];
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void scan_file( void )
  {
  int ch;
  int nr;

  if (!strcmp(inp_name, "-"))
    inp = stdin;
  else
    {
    inp = fopen( inp_name, "r" );
    if (inp == NULL)
      {
      fprintf( stderr, "Can not open input file: %s\a\n", inp_name );
      exit( 2 );
      }
    }

  for (ch=0; ch <256; ch++)
    frequency[ch] = 0;

  nr = 0;
  ch = fgetc( inp );
  while (ch != EOF)
    {
    frequency[ch]++;
    buffer[nr++] = ch;

    if (nr >= MAX_SEQ)
      run_chr_tsts( nr );

    if (nr >= MAX_BUFF)
      {
      shift_buffer( nr );
      nr -= MAX_BUFF;
      }
      
    ch = fgetc( inp );
    }

  shift_buffer( nr );

  for (nr=0; nr<MAX_SEQ; nr++)
    buffer[ nr + MAX_SEQ ] = 0;

  for (nr=MAX_SEQ; nr<MAX_SEQ+MAX_SEQ; nr++)
    run_chr_tsts( nr );
    
  fclose( inp );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int find_best_code( void )
  {
  int t;
  int best;
  int best_t;
  int res;

  best   = -1;
  best_t = -1;
  for (t=0; t<MAX_TEST; t++)
    {
    if (tests[ t ].freq_tst != NULL)
      {
/*      printf( "Testing %s\n", tests[ t ].name );*/
      res = (*tests[ t ].freq_tst)();
      if (res > best)
	{
	best_t = t;
	best   = res;
	}
      if ((verbose_flag > 0) && (res > 0))
	printf( "Test %-40s OK (quality=%d)\n",
		      tests[ t ].name, res );
      }
    }

  if (best == -1)
    best_t = TYPE_ISO_8859_2;	/* Fallback code */

  if (best_t == TYPE_PREFIX)
    {
    inp_type = TYPE_UNSPECIFIED;
    set_prefix( inp_prefix, inp_name );
    if ((verbose_flag > 0) || (out_name == NULL))
      printf( "Best code is: %s using %c (quality=%d)\n",
			     tests[ best_t ].name, inp_prefix, best );
    }
  else if (best_t >= 0)
    {
    inp_type = TYPE_UNSPECIFIED;
    if (tests[best_t].set != NULL)
      (*tests[best_t].set)( inp_name );
    if ((verbose_flag > 0) || (out_name == NULL))
      printf( "Best code is: %s (quality=%d)\n",
			     tests[ best_t ].name, best );
    }

  return best_t;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int  run_test( void )
  {
  int ch;

  scan_file();

  if (verbose_flag > 2)
    for (ch=0; ch <128; ch++)
      {
      if (frequency[ch] != 0)
	printf( "Frequency[%02x]=%5ld\n", ch, frequency[ch] );
      }

  if (verbose_flag > 1)
    for (ch=128; ch <256; ch++)
      {
      if (frequency[ch] != 0)
	printf( "Frequency[%02x]=%5ld\n", ch, frequency[ch] );
      }

  return find_best_code();
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_test( void )
  {
  tests[ TYPE_TEST ].key		= "-t";
  tests[ TYPE_TEST ].name		= "Test";
  tests[ TYPE_TEST ].set		= set_test;
  tests[ TYPE_TEST ].char_tst	= NULL;
  tests[ TYPE_TEST ].freq_tst	= NULL;
  }

/*********************************************************************/
