/*********************************************************************/
/* unicode.c    v 1.0      by J.Sobczyk IAPW       1995.10.20  19:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* UNICODE                                                           */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
char unicode_tbl[2*POL_NR][3] = /* Unicode */
  {
    { '\x01', '\x05',  'a' },	/* LATIN SMALL LETTER A WITH OGONEK */
    { '\x01', '\x07',  'c' },	/* LATIN SMALL LETTER C WITH ACUTE */
    { '\x01', '\x19',  'e' },	/* LATIN SMALL LETTER E WITH OGONEK */
    { '\x01', '\x42',  'l' },	/* LATIN SMALL LETTER L WITH STROKE */
    { '\x01', '\x44',  'n' },	/* LATIN SMALL LETTER N WITH ACUTE */
    { '\x00', '\xF3',  'o' },	/* LATIN SMALL LETTER O WITH ACUTE */
    { '\x01', '\x5B',  's' },	/* LATIN SMALL LETTER S WITH ACUTE */
    { '\x01', '\x7A',  'x' },	/* LATIN SMALL LETTER Z WITH ACUTE */
    { '\x01', '\x7C',  'z' },	/* LATIN SMALL LETTER Z WITH DOT ABOVE */
    { '\x01', '\x04',  'A' },	/* LATIN CAPITAL LETTER A WITH OGONEK */
    { '\x01', '\x06',  'C' },	/* LATIN CAPITAL LETTER C WITH ACUTE */
    { '\x01', '\x18',  'E' },	/* LATIN CAPITAL LETTER E WITH OGONEK */
    { '\x01', '\x41',  'L' },	/* LATIN CAPITAL LETTER L WITH STROKE */
    { '\x01', '\x43',  'N' },	/* LATIN CAPITAL LETTER N WITH ACUTE */
    { '\x00', '\xD3',  'O' },	/* LATIN CAPITAL LETTER O WITH ACUTE */
    { '\x01', '\x5A',  'S' },	/* LATIN CAPITAL LETTER S WITH ACUTE */
    { '\x01', '\x79',  'X' },	/* LATIN CAPITAL LETTER Z WITH ACUTE */
    { '\x01', '\x7B',  'Z' }	/* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  };

long int unicode_hits  = 0;
long int unicode_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int unicode_get( void )
  {
  int	ch1;
  int	ch0;
  int	i;

  ch0 = getc( inp );

  if (ch0 == EOF)
    return ch0;

  ch1 = getc( inp );
  
  for (i=0; i< 2*POL_NR; i++)
    if (((char) ch0 == unicode_tbl[i][0]) &&
        ((char) ch1 == unicode_tbl[i][1])   )
      {
      return POL_CHAR | unicode_tbl[i][2];
      }

  if (ch0 != '\x00')
    {
    fprintf( stderr, "Character %02x%02x converted into @\n", ch0, ch1 );
    ch1 = '@';
    }
  return ch1;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void unicode_put( int ch )
  {
  int i;

  if (ch == EOF)
    ;
  else if (ch & POL_CHAR)
    {
    ch &= 0x0FF;
    for (i=0; i< 2*POL_NR; i++)
      if ((char) ch == unicode_tbl[i][2])
	{
	fputc( unicode_tbl[i][0], out );
	fputc( unicode_tbl[i][1], out );
	}
    }
  else
    {
    fputc( '\x00', out );
    fputc( ch, out );
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_unicode( char *name )
  {
  if      (inp_type == TYPE_UNSPECIFIED)
    {
    inp_type   = TYPE_UNICODE;
    inp_name   = name;
    inp_get    = unicode_get;
    }
  else if (out_type == TYPE_UNSPECIFIED)
    {
    out_type   = TYPE_UNICODE;
    out_name   = name;
    out_put    = unicode_put;
    }
  else
    {
    fprintf( stderr, "Too many switches\a\n" );
    help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_unicode( char *buff )
  {
  int i;

  for (i=0; i< 2*POL_NR; i++)
    if (((char) buff[0] == unicode_tbl[i][0]) &&
        ((char) buff[1] == unicode_tbl[i][1])   )
      {
      unicode_hits++;
      return;
      }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_unicode( void )
  {
  return unicode_hits-unicode_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_unicode( void )
  {
  tests[ TYPE_UNICODE ].key		= "-un";
  tests[ TYPE_UNICODE ].name		= "Unicode";
  tests[ TYPE_UNICODE ].set		= set_unicode;
  tests[ TYPE_UNICODE ].char_tst	= test_char_unicode;
  tests[ TYPE_UNICODE ].freq_tst	= test_freq_unicode;
  }

/*********************************************************************/
